//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// CRC generator
//
module crc_gen (
  clk_i, rst_i, init_i, frame_data_i, data_en_i, crc_rd_i,
  crc_o
  );
  input clk_i;                // Input clock
  input rst_i;                // Input reset signal, asserted on the posedge
  input init_i;               // Initialization input, similar to reset
  input [7:0] frame_data_i;   // Input data for CRC calculation, supplied on the posedge
  input data_en_i;            // Signal to enable CRC calculation, asserted on the posedge
  input crc_rd_i;             // Signal to read 32-bit CRC value, asserted on the posedge
  output [7:0] crc_o;         // Output CRC, data output on the posedge
  
  // Internal wires and registers
  reg [31:0] crc_reg;
  //
  // Input data width is 8bit, and the first bit is bit[0]
  function[31:0] NextCRC;
    input [7:0] D;
    input [31:0] C;
    reg [31:0] NewCRC;
    begin
      NewCRC[0]=C[24]^C[30]^D[1]^D[7];
      NewCRC[1]=C[25]^C[31]^D[0]^D[6]^C[24]^C[30]^D[1]^D[7];
      NewCRC[2]=C[26]^D[5]^C[25]^C[31]^D[0]^D[6]^C[24]^C[30]^D[1]^D[7];
      NewCRC[3]=C[27]^D[4]^C[26]^D[5]^C[25]^C[31]^D[0]^D[6];
      NewCRC[4]=C[28]^D[3]^C[27]^D[4]^C[26]^D[5]^C[24]^C[30]^D[1]^D[7];
      NewCRC[5]=C[29]^D[2]^C[28]^D[3]^C[27]^D[4]^C[25]^C[31]^D[0]^D[6]^C[24]^C[30]^D[1]^D[7];
      NewCRC[6]=C[30]^D[1]^C[29]^D[2]^C[28]^D[3]^C[26]^D[5]^C[25]^C[31]^D[0]^D[6];
      NewCRC[7]=C[31]^D[0]^C[29]^D[2]^C[27]^D[4]^C[26]^D[5]^C[24]^D[7];
      NewCRC[8]=C[0]^C[28]^D[3]^C[27]^D[4]^C[25]^D[6]^C[24]^D[7];
      NewCRC[9]=C[1]^C[29]^D[2]^C[28]^D[3]^C[26]^D[5]^C[25]^D[6];
      NewCRC[10]=C[2]^C[29]^D[2]^C[27]^D[4]^C[26]^D[5]^C[24]^D[7];
      NewCRC[11]=C[3]^C[28]^D[3]^C[27]^D[4]^C[25]^D[6]^C[24]^D[7];
      NewCRC[12]=C[4]^C[29]^D[2]^C[28]^D[3]^C[26]^D[5]^C[25]^D[6]^C[24]^C[30]^D[1]^D[7];
      NewCRC[13]=C[5]^C[30]^D[1]^C[29]^D[2]^C[27]^D[4]^C[26]^D[5]^C[25]^C[31]^D[0]^D[6];
      NewCRC[14]=C[6]^C[31]^D[0]^C[30]^D[1]^C[28]^D[3]^C[27]^D[4]^C[26]^D[5];
      NewCRC[15]=C[7]^C[31]^D[0]^C[29]^D[2]^C[28]^D[3]^C[27]^D[4];
      NewCRC[16]=C[8]^C[29]^D[2]^C[28]^D[3]^C[24]^D[7];
      NewCRC[17]=C[9]^C[30]^D[1]^C[29]^D[2]^C[25]^D[6];
      NewCRC[18]=C[10]^C[31]^D[0]^C[30]^D[1]^C[26]^D[5];
      NewCRC[19]=C[11]^C[31]^D[0]^C[27]^D[4];
      NewCRC[20]=C[12]^C[28]^D[3];
      NewCRC[21]=C[13]^C[29]^D[2];
      NewCRC[22]=C[14]^C[24]^D[7];
      NewCRC[23]=C[15]^C[25]^D[6]^C[24]^C[30]^D[1]^D[7];
      NewCRC[24]=C[16]^C[26]^D[5]^C[25]^C[31]^D[0]^D[6];
      NewCRC[25]=C[17]^C[27]^D[4]^C[26]^D[5];
      NewCRC[26]=C[18]^C[28]^D[3]^C[27]^D[4]^C[24]^C[30]^D[1]^D[7];
      NewCRC[27]=C[19]^C[29]^D[2]^C[28]^D[3]^C[25]^C[31]^D[0]^D[6];
      NewCRC[28]=C[20]^C[30]^D[1]^C[29]^D[2]^C[26]^D[5];
      NewCRC[29]=C[21]^C[31]^D[0]^C[30]^D[1]^C[27]^D[4];
      NewCRC[30]=C[22]^C[31]^D[0]^C[28]^D[3];
      NewCRC[31]=C[23]^C[29]^D[2];
      NextCRC=NewCRC;
    end 
  endfunction
  
  assign crc_o = ~{crc_reg[24],crc_reg[25],crc_reg[26],crc_reg[27],crc_reg[28],crc_reg[29],
                    crc_reg[30],crc_reg[31]};



  always @ (negedge clk_i)
    begin
      if (!rst_i)
        begin
          crc_reg <= 32'hFFFFFFFF;
        end
      else if (init_i)
        begin
          crc_reg <= 32'hFFFFFFFF;
        end
      else if (data_en_i)
        begin
          crc_reg <= NextCRC(frame_data_i,crc_reg);
        end
      else if (crc_rd_i)
        begin
            crc_reg <= {crc_reg[23:0],8'hFF};
        end
    end
    
    
  
endmodule