//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Top module for the memory controller
// 
module mem_ctrl (
  clk_i, rst_i,
  // Wishbone interface
  wb_adr_i, wb_data_i, wb_we_i, wb_sel_i, wb_stb_i, wb_cyc_i,
  wb_data_o, wb_ack_o
  );
  
  input clk_i;          // Input clock signal
  input rst_i;          // Input reset signal
  // Wishbone interface
  input [31:0] wb_adr_i;          // Input address
  input [31:0] wb_data_i;         // Input data
  input wb_we_i;                  // Write enable
  input [3:0] wb_sel_i;           // Byte select signals
  input wb_stb_i;                 // Strobe signal
  input wb_cyc_i;                 // Start cycle 
  output [31:0] wb_data_o;        // Output data
  output wb_ack_o;                // Output acknowledge signal
  
  reg wb_ack_o;
  
  // Internal wires and registers
  reg we_r;
  wire [31:0] wb_data_w;
  
  assign wb_data_o = ({wb_we_i,wb_stb_i,wb_cyc_i} == 3'b011) ? wb_data_w : 32'hz;
  
  // Instantiate modules
  memory m_sram (
    .clk_i        (clk_i),
    .we_i         (we_r), 
    .data_i       (wb_data_i), 
    .be_i         (wb_sel_i), 
    .adr_i        (wb_adr_i[14:2]),
    .data_o       (wb_data_w)
    );
    
  always @ (negedge clk_i)
    begin
      if (!rst_i)
        begin
          we_r <= 1'b0;  
          wb_ack_o <= 1'b0;
        end
      else
        begin
          case ({wb_we_i,wb_stb_i,wb_cyc_i})        // Synthesis full case
            3'b111:
              begin
                we_r <= 1'b1;
                wb_ack_o <= 1'b1;
              end
            3'b011:
              begin
                we_r <= 1'b0;
                wb_ack_o <= 1'b1;
              end
            default:
              begin
                we_r <= 1'b0;
                wb_ack_o <= 1'b0;
              end
          endcase
        end
    end
    
    
endmodule