//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Operational registers file
// 
module ops_regs (
  clk_i, rst_i, data_i, adr_i, we_i,
  cr_o, txdp_o, txcfg_o, rxcfg_o, rxdp_o,
  rfcr_o, txdpw_o, rxdpw_o
  
  );
  input clk_i;                        // Input clock signal
  input rst_i;                        // Input reset signal
  input [31:0] data_i;                // Input data to update registers
  input [7:0] adr_i;                  // Input address to update registers
  input we_i;                         // Write enable signal to update registers
  output [31:0] cr_o;                 // Output command register
  output [31:0] txdp_o;               // Tx descriptor pointer
  output [31:0] txcfg_o;              // Output transmit configuration register 
  output [31:0] rxcfg_o;              // Output receive configuration register
  output [31:0] rxdp_o;               // Rx descriptor pointer
  output [31:0] rfcr_o;               // Receive filtering control
  output txdpw_o;                     // Output signal for tx_bufmgr that txdp has been updated
  output rxdpw_o;                     // Output signal for rx_bufmgr that rxdp has been updated
    
  // Internal registers and wires
  reg [31:0] cr;                      // Command register. Offset 0000h, R/W
  reg [31:0] cfg;                     // Configuration and media status register. Offset 0004h, R/W
  reg [31:0] txdp;                    // Ttransmit Descriptor Pointer, offset 0020h, R/W
  reg [31:0] txcfg;                   // Transmit configuration register. Offset 0028h, R/W
  reg [31:0] rxdp;                    // Receive descriptor pointer, offset 0030h, R/W
  reg [31:0] rxcfg;                   // Receive configuration register. Offset 0038h, R/W
  reg [31:0] rfcr;                    // Receive filtering conrol, offset 0048h, R/W
  reg txdpw_o;
  reg rxdpw_o;
  
  assign cr_o = cr;
  assign txdp_o = txdp;
  assign txcfg_o = txcfg; 
  assign rxcfg_o = rxcfg;
  assign rxdp_o = rxdp;
  assign rfcr_o = rfcr;
  
  always @ (posedge clk_i)
    begin
      if (!rst_i)       // Reset condition
        begin
          cr <= 32'h0;
          cfg <= 32'h0;
          txcfg <= 32'h00000110;
          rxcfg <= 32'h00000004;
          txdp <= 32'h300;
          rxdp <= 32'h200;
          rfcr <= 32'h0;
          txdpw_o <= 1'b0;
          rxdpw_o <= 1'b0;
        end
      else
        begin
          if (we_i)
            begin
              case (adr_i)            // Synthesis full case
                8'h0:   cr <= data_i;
                8'h20: txdp <= data_i;
                8'h28: txcfg <= data_i;
                8'h30: rxdp <= data_i;
                8'h38: rxcfg <= data_i;
                8'h48: rfcr <= data_i;
              endcase
            end
          else
            begin
              
            end
            
          if (we_i && (adr_i == 8'h20))
            txdpw_o <= 1'b1;
          else
            txdpw_o <= 1'b0;
            
          if (we_i && (adr_i == 8'h30))
            rxdpw_o <= 1'b1;
          else 
            rxdpw_o <= 1'b0;
        end
    end
endmodule