//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Parameters file

`define TX_FIFO_SIZE      16'h0800     // FIFO size in 32 bit words (2K)
`define FIFO_AFULL        11'h600      // Limit for FIFO almost full signal - to make sure one more packet
                                       // can be received
`define FIFO_AEMPTY       11'h20       // Limit for FIFO almost empty signal

// Commands of TX FSM
`define TX_NOP            4'b0000       // IDLE - no operations
`define TX_DREAD          4'b0001       // Command  to read one descriptor
`define TX_FBLOCK         4'b0010       // Waiting for free space in the Tx FIFO
`define TX_FREAD          4'b0011       // Transferr of fragment from WB to the Tx FIFO
`define TX_DADVANCE       4'b0100       // Update descriptor in the main memory after Tx is completed
// Index into the states register of TX FSM
`define TX_IDLE           3'b000
`define TX_DESC_READ      3'b001
`define TX_FIFO_BLOCK     3'b010
`define TX_FRAG_READ      3'b011
`define TX_ADVANCE        3'b100

// States of the wishbone controller FSM
`define WB_IDLE          4'b0000           // Idle
`define WB_BREAD         4'b0001           // Block read
`define WB_BWRITE        4'b0010           // Block write
`define WB_EXWRITE       4'b0101           // External host writes  words to the controller (one or more)

// Number of bytes to be read from memory
`define  RD_DESC        16'h10         // Tx/Rx descriptor consists of 4 32-bit words
`define  WB_BASE        32'hFFF80000   // Base in WB address space

// Tx MAC FSM states
`define TXMAC_IDLE      4'b0000           // Tx MAC is idle
`define TXMAC_START     4'b0001           // Start of data transmission
`define TXMAC_PREAMB    4'b0010           // Transmission of preamble
`define TXMAC_FDSTART   4'b0011           // Start of data transmission in full duplex mode - no collision detection
`define TXMAC_FCS       4'b0100           // Transmission of CRC
`define TXMAC_PAD       4'b0101           // Transmission of padding
`define TXMAC_IFRAME    4'b0110           // Interframe gap


`define PREAMBLE_PATTERN    8'h55        // 8'hAA
`define SFD_PATTERN         8'hD5        // 8'hAB
`define PAD_PATTERN         8'hFF
`define MIN_PACKET_SIZE     16'h3B          // Size of minimum Ethernet packet (parameter is actually = 59
                                            // while minimum packet size is 60
                                            
`define IFRAME_GAP1000    16'h10    // Test assignment    
//`define IFRAME_GAP1000      16'h10          // Interfame gap for 1GHz network
`define IFRAME_GAP100       16'h10          // Interfame gap for 100MHz network 

// Rx MAC FSM states
`define RXMAC_IDLE      4'b0000             // Rx MAC is idle
`define RXMAC_START     4'b0001             // Start of data reception
`define RXMAC_FILTER    4'b0010             // Data filtering
`define RXMAC_FDDATA    4'b0011             // Reception of main data frame in duplex mode
`define RXMAC_FDDC      4'b0100             // Data reception of the frame is completed
`define RXMAC_CRC       4'b0101             // CRC check
`define RXMAC_FFULL     4'b0110             // FIFO is almost full - packet dropped
`define RXMAC_DVL       4'b0111             // Data valid has been lost during packet reception

// Rx buffer manager FSM states
`define RX_IDLE         4'b0000             // Idle
`define RX_DESC_READ    4'b0001             // Descriptor read
`define RX_FIFO_BLOCK   4'b0010             // Prepare FIFO block to be written
`define RX_FRAG_WRITE   4'b0011             // Write on FIFO block
`define RX_ADVANCE      4'b0100             // Update Rx descriptor
`define RX_SCROLL       4'b0101             // Data packet has CRC error - just scroll FIFO and not write to the main memory

`define RX_TIMER_EXP    16'h0100            // Time to wait during polling of rx descriptor
`define TX_TIMER_EXP    16'h0100            // Time to wait during polling of tx descriptor

