//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
		 
// Test module to capture Tx'd data
//
 
`include "parms.v"
//
module txd_capture (
 tx_clk_i, gp_clk_i, rst_i, txd_i, txen_i, test01_i, test02_i, 
 data_o, we_o
);
  input tx_clk_i;                // Input Tx clock
  input gp_clk_i;                // Main system clock
  input rst_i;                   // Input reset signal
  input [7:0] txd_i;             // Input data
  input txen_i;                  // Input Tx enable signal
  input [7:0] test01_i;          // General purpose test input
	 
  input [15:0] test02_i;         // General purpose test input
  output [7:0] data_o;           // Data to be sent out to the UART
  output we_o;                   // Write enable signal for UART
  
  reg we_o;
  reg [7:0] data_o;
   
  // Internal wires and registers
  reg [7:0] mem_buf [0:511];   // Memory buffer
  reg [7:0] index_buf [0:511]; // Buffer for index
  reg [7:0] array01 [0:511];
  reg [7:0] array02 [0:511];
  reg [8:0] adrw;              // Write address
  reg [8:0] adrr;              // Read address
  reg [7:0] index;
	  
  reg [1:0] incr;
								  
  reg [31:0] txen_counter;
 
  reg start_count = 1'b0;
  
  always @ (posedge tx_clk_i)
    begin
      if (~rst_i)
        begin
          adrw <= 9'h0;
          index <= 8'h0;
          start_count <= 1'b0;
          txen_counter <= 32'h0;
        end
      else
        begin
				  
          if (test01_i[3:0] ==  `TXMAC_START)
            start_count <= 1'b1;
          if (start_count && (txen_counter < 32'h78))
            begin
              mem_buf[adrw] <= txd_i;
              index_buf[adrw] <= test01_i;
              array01[adrw] <= test02_i[7:0];
              array02[adrw] <= {7'b0,txen_i};
              adrw <= adrw + 1'b1;
              index <= index + 1'b1;
            end
				
          if (start_count && (txen_counter < 32'h80))
            txen_counter <= txen_counter + 1'b1;
        end
    end 
    

    
   always @ (posedge gp_clk_i)
     begin
       if (~rst_i)
         begin
           adrr <= 9'h0;
           we_o <= 1'b0;
  
           incr <= 2'b00;
         end
       else
         begin
           if (~txen_i && (adrr != adrw))
             begin
		
               we_o <= 1'b1;
               incr <= incr + 1'b1;
               case (incr)                
                 2'b00: data_o <= mem_buf[adrr];
                 2'b01: data_o <= index_buf[adrr];
                 2'b10: data_o <= array02[adrr];
                 2'b11:
                   begin
                     data_o <= array01[adrr];
                     adrr <= adrr + 1'b1;
                   end
               endcase
             end			
           else
             begin
               we_o <= 1'b0;
             end
         end
     end
  
endmodule