//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Top module for the Wishbone host
//

`include "parms.v"
module wb_host (
  clk_i, rst_i,
  // Wishbone interface with gigabit ethernet controller
  wb_adr_i, wb_data_i, wb_we_i, wb_sel_i, wb_stb_i, wb_ack_i,
  wb_cyc_i, wb_tagn_i,
  wb_adr_o, wb_data_o, wb_we_o, wb_sel_o, wb_stb_o, wb_ack_o,
  wb_cyc_o, wb_tagn_o,
  // Wishbone interface with CPU
  wbcp_adr_i, wbcp_data_i, wbcp_we_i, wbcp_sel_i, wbcp_stb_i, wbcp_ack_i,
  wbcp_cyc_i, wbcp_tagn_i,
  // Specific part of wishbone interface with memory controller
  wbm_data_i, wbm_ack_i, 
  wbm_cyc_o, wbm_data_o
  );
  
  input clk_i;            // Input clock
  input rst_i;            // Input reset signal
  
  // Wishbone interface
  input [31:0] wb_adr_i;            // Input address
  input [31:0] wb_data_i;           // Input data
  input wb_we_i;                    // Write enable signal
  input [3:0] wb_sel_i;             // Byte select signal
  input  wb_stb_i;                  // Strobe input - select signal
  input wb_ack_i;                   // Acknowledge signal
  input wb_cyc_i;                   // Bus cycle indicator
  input wb_tagn_i;                  // Tag input signal
  output [31:0] wb_adr_o;           // Output address
  output [31:0] wb_data_o;          // Output data
  output wb_we_o;                   // Output write enable signal
  output [3:0] wb_sel_o;            // Byte select signal
  output wb_stb_o;                  // Strobe output
  output wb_ack_o;                  // Acknowledge signal
  output wb_cyc_o;                  // Bus cycle indicator
  output wb_tagn_o;                 // Tag output signal
  
  input [31:0] wbcp_adr_i;          // Input address from CPU
  input [31:0] wbcp_data_i;         // Input data from CPU
  input wbcp_we_i;                  // Input write enable signal from CPU
  input [3:0] wbcp_sel_i;           // Byte select signal from CPU
  input wbcp_stb_i;                 // Strobe input - select signal from CPU
  input wbcp_ack_i;                 // Acknowledge signal from CPU
  input wbcp_cyc_i;                 // Bus cycle indicator from CPU
  input wbcp_tagn_i;                // Tag input signal from CPU
  
  input [31:0] wbm_data_i;          // Input data from the memory controller
  input wbm_ack_i;                  // Input ack signal from the memory controller
  output wbm_cyc_o;                 // Output strobe signal to the memory controller  
  output [31:0] wbm_data_o;         // Output data to the memory controller                
  
  reg [31:0] wbh_adr;   // Address generated internally by the host controller
  reg [31:0] wbh_data;  // Data generated internally by the host controller
  reg wbh_we;
  reg [3:0] wbh_sel;
  reg wbh_stb;
  reg wbh_cyc;

  reg [3:0] wbhost_flag;
  wire wbh_cyc_w;
  wire [31:0] wb_data_m;
  reg [31:0] wb_adro_neg;
  reg wb_cyci_neg;


// Control signals multiplexers
  assign wb_adr_o = (wb_cyc_i == 1'b1) ? wb_adr_i :
                    ({wb_cyc_i,wbcp_cyc_i} == 2'b01) ? wbcp_adr_i : wbh_adr;
  assign wb_data_o = (wb_adr_o[31:28] == 4'h0) ? wbm_data_i: wbcp_data_i;
  assign wb_stb_o = (wb_cyc_i == 1'b1) ? wb_stb_i :
                    ({wb_cyc_i,wbcp_cyc_i} == 2'b01) ? wbcp_stb_i : wbh_stb;
  assign wb_we_o = (wb_cyc_i == 1'b1) ? wb_we_i :
                    ({wb_cyc_i,wbcp_cyc_i} == 2'b01) ? wbcp_we_i : wbh_we;
  assign wb_sel_o = (wb_cyc_i == 1'b1) ? wb_sel_i :
                     ({wb_cyc_i,wbcp_cyc_i} == 2'b01) ? wbcp_sel_i : wbh_sel;
  assign wbh_cyc_w = (wb_cyc_i == 1'b1) ? wb_cyc_i :
                     ({wb_cyc_i,wbcp_cyc_i} == 2'b01) ? wbcp_cyc_i : wbh_cyc;
  assign wbm_cyc_o = (wb_adr_o[31:28] == 4'h0) ? wbh_cyc_w : 1'b0;
  assign wb_cyc_o = (wb_adr_o[31:28] == 4'hF) ? wbh_cyc_w : 1'b0; 
  assign wb_ack_o = (wb_adro_neg[31:28] == 4'h0) ? wbm_ack_i : wb_ack_i; 
  assign wb_data_m = (wb_cyci_neg == 1'b1) ? wb_data_i : wbcp_data_i;
  assign wbm_data_o = (wb_adr_i[31:28] == 4'h0) ? wb_data_m: wbh_data;
  
  


 always @ (negedge clk_i)
   begin
     wb_adro_neg <= wb_adr_o;
     wb_cyci_neg <= wb_cyc_i;
   end
    
    
endmodule
